const fs = require("fs");
const path = require("path");
const createDir = require("../utils/createDir");

const getDirName = function(filePath) {
  const dirPath = path.dirname(filePath);
  const dirNames = dirPath.split(path.sep);
  return dirNames[dirNames.length - 1];
}

const createRouterJsonFile = async (content, config) => {
  const routerJsonFilePath = path.join(config.packagePath, 'src' ,'router', 'router.json');
    let routerData = [];
    if(fs.existsSync(routerJsonFilePath)) {
      const fileData = await fs.readFileSync(routerJsonFilePath, 'utf-8');
      pageData = new Function(`return ${fileData}`)();
      routerData = pageData || [];
    }
    const dirs = fs.readdirSync(config.pagesPath);
    const pages = [];
    dirs.forEach((dirName) => {
      const filePath = path.join(config.pagesPath, dirName, 'icode.page.json');
      if(fs.existsSync(filePath)) {
        const pageName = getDirName(filePath);
        pages.push(pageName);
      }
    })

    const newRouterData = [];
    let isHadRouterItem = false;
    const currComponentName = content.pageName;
    routerData.forEach(routerItem => {
      if(pages.indexOf(routerItem.componentName) >= 0) {
        newRouterData.push(routerItem);

        if(currComponentName === routerItem.componentName) {
          isHadRouterItem = true;
        }
      }
      
    });

    if(!isHadRouterItem) {
      newRouterData.push({
        path: `/${currComponentName}`,
        title: `${currComponentName}`,
        componentName: `${currComponentName}`
      });
    }

    await fs.writeFileSync(
      routerJsonFilePath,
      JSON.stringify(newRouterData),
      {
        encoding: "utf-8",
      }
    );

}

const templateData = [
  './.vscode/settings.json',
  './assets/.txt',
  './src/layouts/index.jsx',
  './src/router/index.js',
  './src/utils/request.js',
  './src/utils/myDva.js',
  './src/bootstrap.js',
  './src/index.ejs',
  './src/index.js',
  './src/style.less',
  '.gitignore___',
  '.prettierignore',
  '.prettierrc.js',
  'README.md',
  'runner.config.js',
];

module.exports = async function (content, config) {
  try {
    let i, k;
    for(i = 0, k = templateData.length; i < k; i++) {
      const templateFilePath = path.join(__dirname, 'template', templateData[i]);
      const targetPath = path.join(config.packagePath, templateData[i].replace(/___/, ''));
      const dirPath = path.dirname(targetPath);
      
      if(!fs.existsSync(dirPath)) {
        await createDir(dirPath);
      }
      if (!fs.existsSync(targetPath)) {
        await fs.copyFileSync(templateFilePath, targetPath);
      }
    }

    // const icodeCliFileTargetPath = path.join(config.packagePath, 'runner.config.js');
    // if (!fs.existsSync(icodeCliFileTargetPath)) {
    //   await fs.copyFileSync(path.join(__dirname, 'template', 'runner.config.js'), icodeCliFileTargetPath);
    // }

    await createRouterJsonFile(JSON.parse(content), config);

  } catch (e) {
    console.log("---------json-2-code layout error=", e);
  }
};