import axios from 'axios';
import qs from 'qs';

const trimParams = params => {
  if (params && params.constructor === Object) {
    Object.keys(params).forEach(key => {
      if (typeof params[key] === 'string') {
        params[key] = params[key].replace(/(^\s+)|(\s+$)/g, '');
      }
    });
  } else if (typeof params === 'string' && params.indexOf('=') > 0) {
    const newParams = qs.parse(params);
    params = trimParams(newParams);
    params = qs.stringify(params);
  }
  return params;
};

const parseData = (res) => {
  return {
    result: res.result,
    data: res.data,
    count: res.count,
    msg: res.msg,
  }
};

const instance = axios.create({
  timeout: 60000,
  headers: { 'X-Requested-With': 'XMLHttpRequest' },
});

instance.interceptors.response.use(
  response => {
    return parseData(response.data);
  },
  error => {
    //
  },
);

const request = async config => {
  config.method = config.method || 'POST';
  config.headers = config.headers || {};

  if (config.type === 'json') {
    config.headers['Content-Type'] = 'application/json;charset=UTF-8';
  } else {
    config.headers['Content-Type'] = 'application/x-www-form-urlencoded';
  }

  try {
    if (config.body) {
      if(config.method === 'GET' && Object.keys(config.body).length) {
        config.url = `${config.url}?${qs.stringify(config.body)}`;
      }
      else if (
        config.body && config.body.constructor === Object &&
        config.headers &&
        config.headers['Content-Type'] &&
        config.headers['Content-Type'].indexOf('application/x-www-form-urlencoded') >= 0
      ) {
        config.body = qs.stringify(config.body);
      }
      config.body = trimParams(config.body);
    }
    const res = await instance({
      ...config,
      data: config.body,
      validateStatus: function(status) {
        // const href = window.location.href;
        // const loginUrl = `${_PROCESS_ENV_BASE_.replace(/\/$/, '')}/login`;
        // if(status === 401 && href.indexOf(loginUrl) < 0) {
        //   window.location.href = loginUrl;
        // }

        return (status >= 200 && status < 300); // default
      },
    });
    return res;
  } catch (e) {
    //
  }
};

export default request;