const fs = require("fs");
const path = require("path");
const ejs = require("ejs");
const {
  init,
  getStaticData,
  getDefaultValue,
  getParamDefaultValueState,
  getApiServiceName,
  getApiStateName,
  setNewNodeId,
  getCurrentRelations,
  setNewDataModelName,
  getNewEvents,
  getNeedUpdateParams,
  getCss,
  getStaticDataName,
  getRefreshDataHtml,
  getNewNodeId,
  getRequestStructuresType,
  saveImgToLocation,
  saveGlobalState,
  replaceImgUrlInCss,
  replaceImgUrlInProps,
  buildComponentDependenciesHtml,
  getCurrApiData,
} = require("./utils");
const {isValue} = require('@icode-run/utils/cjs/utils');
const createDir = require("../utils/createDir");

const contentInfo = function ({
  data, 
  resourcesPath, 
  imagesDirPath, 
  globalDirPath, 
  packagePath,
  isLoadingNpm,
}) {
  const dataString = JSON.stringify(data);
  init();
  this.updateComponentNodeId(data.componentConfig);
  const dataModelsArray = setNewDataModelName(data.dataModels);

  this.imagesDirPath = imagesDirPath;
  this.globalDirPath = globalDirPath;
  this.data = data;
  this.relations = data.relations || {};
  this.dataModels = data.dataModels || {};
  this.dataModelEvents = {};
  // this.remoteEntryPaths = {};
  this.dependencies = {};
  this.style = {};
  this.css = {};
  this.propsOptions = {};
  this.propsValues = {};
  this.paramDefaultValueState = {}; // 组件中param的属性的默认值{componentName: {propName: '值'}}
  this.staticData = {}; // 收集使用到的静态数据, 整理后，单独放入一个文件，引入到state中
  this.refreshData = {}; // 收集需要更新数据的配置
  this.codeConfig = {};
  this.useDependencies = {};

  this.renderHtml = this.getRenderHtml(data.componentConfig);
  const dependenciesHtml = buildComponentDependenciesHtml({
    packagePath,
    isLoadingNpm,
    dependencies: this.dependencies,
    versions: data.versions || {},
    remoteEntryPaths: data.remoteEntryPaths,
  })

  const relationsDataModels = [];
  const apiParamsConfig = {};
  const needUpdateParams = {};
  const needNumberKeyNames = {};
  const apiParamDefaultValueStates = [],
    apiDataConfig = {};
  const relations = data.relations || {};
  let nodeId;
  const useDataModels = [];
  for(nodeId in relations) {
    const relation = relations[nodeId];
    if(relation && relation.events && relation.events.length) {
      const newEventsData = getNewEvents({
        events: relation.events, 
        dataModels: this.dataModels, 
        relations: this.relations,
        paramDefaultValueState: this.paramDefaultValueState,
      }) || {};
      const newEvents = newEventsData.events;
      let eventNames = [];
      if(relation.submitDataModelName) {
        eventNames = [].concat(eventNames, ["submitSuccess", "submitError", "submitComplete"]);
      }
      if(relation.dataSourceType === 'api' && relation.dataSourceName) {
        eventNames = [].concat(eventNames, ["fetchSuccess", "fetchError", "fetchComplete"]);
      }
      eventNames.forEach(eventName => {
        const serviceName = getApiServiceName(this.dataModels[relation.submitDataModelName]);
        if(newEvents[eventName]) {
          this.dataModelEvents[serviceName] = this.dataModelEvents[serviceName] || {};
          this.dataModelEvents[serviceName][eventName] = newEvents[eventName];
        }
      })
    }
    if (relation.dataSourceType && relation.dataSourceName) {
      useDataModels.push({
        ...this.dataModels[relation.dataSourceName],
        dataModelName: relation.dataSourceName,
      })
    }
    if(relation.submitDataModelName) {
      useDataModels.push({
        ...this.dataModels[relation.submitDataModelName],
        dataModelName: relation.submitDataModelName,
      })
    }
  }

  useDataModels.map(dataModel => {
    const serviceName = getApiServiceName(dataModel);
    relationsDataModels.push({
      serviceName: serviceName,
      serviceDataPrefix: getApiStateName(dataModel.nodeId, {
        relations: this.relations,
        dataModels: this.dataModels,
        from: dataModel.type,
      }),
      ...dataModel,
    });

    const formatConfigs = {};
    const apiParams = dataModel.apiParams || {};
    const paramStateName = `${getApiStateName(dataModel.nodeId, {
      relations: this.relations,
      dataModels: this.dataModels,
      from: dataModel.type,
    })}Params`;
    let keyName;
    for (keyName in apiParams) {
      // 格式化
      const formatConfig =
        (apiParams[keyName] && apiParams[keyName].formatConfig) || {};
      const defaultValueConfig =
        (apiParams[keyName] && apiParams[keyName].defaultValueConfig) || {};
      const isFormatConfig =
        Object.values(formatConfig) && Object.values(formatConfig).length;
      if (isFormatConfig) {
        formatConfigs[keyName] = formatConfig;
      }

      // 参数默认值, 过滤掉组件已设置的默认值(即使是参数项中设置了默认值，也只会用组件中设置的默认值)
      const newDefaultValue = getDefaultValue(keyName, defaultValueConfig);
      if (
        newDefaultValue &&
        !(
          this.paramDefaultValueState[paramStateName] &&
          this.paramDefaultValueState[paramStateName][keyName]
        )
      ) {
        this.paramDefaultValueState[paramStateName] =
          this.paramDefaultValueState[paramStateName] || {};
        this.paramDefaultValueState[paramStateName][keyName] =
          newDefaultValue;
      }
    }

    // 接口请求成功后，更新相关联的“其它接口参数默认值”的当前值
    needUpdateParams[serviceName] = getNeedUpdateParams(dataModel.dataModelName, {
      dataModels: this.dataModels,
    })

    // 格式化-请求参数
    if (Object.values(formatConfigs) && Object.values(formatConfigs).length) {
      apiParamsConfig[serviceName] = formatConfigs;
    }

    // 获取数值格式化字段
    const requestStructures = dataModel.requestStructures || [];
    requestStructures.forEach(structure => {
      if(structure.type === 'number') {
        needNumberKeyNames[serviceName] = needNumberKeyNames[serviceName] || []
        needNumberKeyNames[serviceName].push(structure.keyName);
      }
    })
    

    // 默认值-请求参数
    // if (currentDefaultValues && currentDefaultValues.length) {
    //   apiParamDefaultValueStates.push({
    //     name: `${getApiStateName(nodeId)}Params`,
    //     children: currentDefaultValues,
    //   });
    // }

    // 格式化-返回数据
    const currentProps = this.propsValues[dataModel.nodeId] || {};
    if (dataModel.apiData) {
      apiDataConfig[serviceName] = {
        apiData: dataModel.apiData || {},
        dimensionKeyName:
          (currentProps.dimensionConfig &&
            currentProps.dimensionConfig[0] &&
            currentProps.dimensionConfig[0].keyName) ||
          "",
        measureConfig:
          currentProps.measureConfig && currentProps.measureConfig.length
            ? currentProps.measureConfig
            : [],
      };
    }
    
  })

  const defaultValueState = Object.keys(this.paramDefaultValueState).map(
    (stateName) => {
      let children = this.paramDefaultValueState[stateName];
      children = children && Object.values(children);
      return {
        name: stateName,
        children: children,
      };
    }
  );
  
  // const newDataModelEvents = {};
  // Object.keys(this.dataModelEvents).forEach(seviceName => {
  //   newDataModelEvents[seviceName] = newDataModelEvents[seviceName] || {};
  //   Object.key(this.dataModelEvents[seviceName]).forEach(eventName => {
  //     newDataModelEvents[eventName] = 
  //   })
    
  // })
  ejs.renderFile(
    path.join(__dirname, "./template/index.ejs"),
    {
      dependenciesHtml: dependenciesHtml || '',
      renderHtml: this.renderHtml,
      dataModels: dataModelsArray,
      relationsDataModels: relationsDataModels,
      globalState: saveGlobalState({
        globalState: data.globalState, 
        globalDirPath: this.globalDirPath
      }),
      defaultValueState: defaultValueState,
      apiParamsConfig: apiParamsConfig,
      apiDataConfig: apiDataConfig,
      dataModelEvents: this.dataModelEvents,
      needUpdateParams: needUpdateParams || {},
      needNumberKeyNames,
      staticDataKeys: Object.keys(this.staticData),
      codeConfigKeys: Object.keys(this.codeConfig),
      refreshDataHtml: getRefreshDataHtml(Object.values(this.refreshData)) || '',
      useDependencies: this.useDependencies || {},
    },
    {},
    function (err, str) {
      fs.writeFileSync(path.join(resourcesPath, "/index.jsx"), str, {
        encoding: "utf-8",
      });
    }
  );

  if(Object.keys(this.staticData).length) {
    ejs.renderFile(
      path.join(__dirname, "./template/staticData.ejs"),
      {
        staticData: this.staticData
      },
      {},
      function (err, str) {
        fs.writeFileSync(path.join(resourcesPath, "/staticData.js"), str, {
          encoding: "utf-8",
        });
      }
    );
  }

  if(Object.keys(this.codeConfig).length) {
    ejs.renderFile(
      path.join(__dirname, "./template/codeConfig.ejs"),
      {
        codeConfig: this.codeConfig
      },
      {},
      function (err, str) {
        fs.writeFileSync(path.join(resourcesPath, "/codeConfig.js"), str, {
          encoding: "utf-8",
        });
      }
    );
  }

  ejs.renderFile(
    path.join(__dirname, "./template/less.ejs"),
    {
      style: this.style,
      css: this.css
    },
    {},
    function (err, str) {
      fs.writeFileSync(path.join(resourcesPath, "/index.module.less"), str, {
        encoding: "utf-8",
      });
    }
  );

  ejs.renderFile(
    path.join(__dirname, "./template/service.ejs"),
    {
      dataModels: dataModelsArray,
    },
    {},
    function (err, str) {
      fs.writeFileSync(path.join(resourcesPath, "/service.js"), str, {
        encoding: "utf-8",
      });
    }
  );

  fs.writeFileSync(
    path.join(resourcesPath, "/icode.page.json"),
    dataString
  );

  
};
contentInfo.prototype = {

  camelToMiddleline: function (camelStr) {
    return camelStr.replace(/[A-Z]/g, function (s) {
      return "-" + s.toLowerCase();
    });
  },

  getPropsData: function (item, isScope) {
    const relations = this.relations;
    const dataModels = this.dataModels;
    let dataPropsValue = "";
    const nodeId = item.nodeId;
    const relation = getCurrentRelations(nodeId, relations);
    const apiData = getCurrApiData({
      nodeId,
      relations,
      dataModels
    }) || {};
    const isHadApiData = Object.keys(apiData).length;
    const dataSourceType = relation.dataSourceType;

    const propsOptions = item.propsOptions || [];
    let dimensionKeyName;
    let measureConfig;
    const props = item.props || {};
    propsOptions.map(item => {
      if(item.type === 'dimension' && props[item.name] && props[item.name][0] && props[item.name][0].keyName) {
        dimensionKeyName = props[item.name][0].keyName;
      }
      if(item.type === 'measure' && props[item.name] && props[item.name][0] && props[item.name].length) {
        measureConfig = props[item.name];
      }
    })

    // 来源于作用局的数据
    if (isScope && dataSourceType === "scope" && relation.scopeNodeId) {
      const name = `this.state.${getApiStateName(relation.scopeNodeId, {relations, dataModels})}Scope`;
      if(isHadApiData) {
        dataPropsValue = `parseApiData(${name}, ${JSON.stringify(apiData)})`;
        this.useDependencies.parseApiData = true;
      }
      else {
        dataPropsValue = `${name}`;
      }
    } else if (dataSourceType === "static" && relation.staticData) {
      const staticData = getStaticData(relation.staticData);
      const staticDataName = getStaticDataName(nodeId);
      this.staticData[staticDataName] = staticData;
      dataPropsValue = `this.state.${staticDataName}`;
      // if(Object.keys(apiData).length) {
      //   apiData = JSON.stringify(apiData);
      //   dataPropsValue = `parseApiData(${JSON.stringify(
      //     staticData
      //   )}, ${apiData})`;
      // }
      // else {
      //   dataPropsValue = `${JSON.stringify(
      //     staticData
      //   )}`;
      // }
      
    }
    // 来源于接口返回的数据
    else if (dataSourceType === "api" && relation.dataSourceName) {
      if((dimensionKeyName && measureConfig) || isHadApiData) {
        dataPropsValue = `parseApiData(this.state.${getApiStateName(relation.nodeId, {dataModels, relations})}Data, ${isHadApiData ? JSON.stringify(apiData) : 'null'}, '${dimensionKeyName || ''}', ${measureConfig && measureConfig.length ? JSON.stringify(measureConfig) : 'null'})`
        this.useDependencies.parseApiData = true;
      }
      else {
        dataPropsValue = `(this.state.${getApiStateName(relation.nodeId, {dataModels, relations})}Data || [])`;
      }
    }
    else if (dataSourceType === "state" && relation.dataSourceName) {
      const currDataModel = dataModels[relation.dataSourceName] || {}
      if((dimensionKeyName && measureConfig) || isHadApiData) {
        dataPropsValue = `parseApiData(this.state.${getApiStateName(currDataModel.nodeId, {dataModels, relations})}Data, ${isHadApiData ? JSON.stringify(apiData) : 'null'}, '${dimensionKeyName || ''}', ${measureConfig && measureConfig.length ? JSON.stringify(measureConfig) : 'null'})`;
        this.useDependencies.parseApiData = true;
      }
      else {
        dataPropsValue = `(this.state.${getApiStateName(currDataModel.nodeId, {dataModels, relations})}Data || [])`;
      }
    }
    else if (dataSourceType === "event" && relation.stateNodeId) {
      dataPropsValue = `(this.state.${getApiStateName(relation.stateNodeId, {dataModels, relations})}EventData || [])`;
    }

    return dataPropsValue;
  },

  // 获取验证规则
  getRules: function(paramPropsOptionValues) {
    const dataModels = this.dataModels || {};
    let rules = [];
    (paramPropsOptionValues || []).map(item => {
      if (
        !(rules && rules.length) &&
        dataModels[item.serviceName] &&
        dataModels[item.serviceName].apiParams &&
        dataModels[item.serviceName].apiParams[item.keyName] &&
        dataModels[item.serviceName].apiParams[item.keyName].validateConfig &&
        dataModels[item.serviceName].apiParams[item.keyName].validateConfig.length
      ) {
        rules =
        dataModels[item.serviceName].apiParams[item.keyName].validateConfig;
      }
    })
    return rules;
  },

  /**
   *
   * @param {*} item 当前组件的componentConfig数据
   * @param {*} isScope 是否为作用域数据
   * @returns
   */
  getProps: function (item, isScope, isFormItem) {
    const relations = this.relations;
    const dataModels = this.dataModels;
    const propsOptionsObj = this.propsOptions[item.nodeId];
    const relation = getCurrentRelations(item.nodeId, relations);
    const defaultProps = item.props;
    let props = "";
    let textProps = null;

    for (let key in defaultProps) {
      if (defaultProps[key]) {
        let value = defaultProps[key];
        if (
          propsOptionsObj &&
          propsOptionsObj[key] &&
          propsOptionsObj[key].type === "param"
        ) {
          const value = propsOptionsObj[key].value;
          const outputType = propsOptionsObj[key].outputType;
          const stateName = `${getApiStateName(item.nodeId, {relations, dataModels})}DefaultValue`;

          if (value && value.length) {
            const rules = this.getRules(value);
            const firstValue = value[0];
            const from = dataModels[firstValue.serviceName] && dataModels[firstValue.serviceName].type;
            // value: this.state.${getApiStateName(
            //   firstValue.nodeId, {relations, dataModels, from}
            // )}Params && this.state.${getApiStateName(
            //   firstValue.nodeId, {relations, dataModels, from}
            // )}Params.${firstValue.keyName},
            const resultValues = {};
            const stateValue = {};
            const stateFetch = {};
            value.forEach(valueItem => {
              const paramStateName = getApiStateName(valueItem.nodeId, {
                relations, 
                dataModels, 
                from: dataModels[valueItem.serviceName] && dataModels[valueItem.serviceName].type
              }) + 'Params';

              const structureType = valueItem.structureType;
              const transformType = valueItem.transformType || {};
              const isTriggerRequest = valueItem.isTriggerRequest;
              const apiParamOutputType = getRequestStructuresType(dataModels[valueItem.serviceName], valueItem.keyName);
              if(apiParamOutputType !== outputType || Object.values(valueItem.transformType || {}).length) {
                resultValues[valueItem.keyName] = `const ${valueItem.keyName} = getOutputValue({
                  outputType: '${apiParamOutputType}',
                  value,
                  ${isValue(transformType.index) ? 'targetIndex: '+transformType.index+',' : ''}
                  ${transformType.join ? 'valueSplit: '+transformType.join : ''}
                });
                `;
                this.useDependencies.getOutputValue = true;
              }

              const paramString = `${valueItem.keyName}: ${resultValues[valueItem.keyName] ? valueItem.keyName : 'value'}`;
              if(isTriggerRequest) {
                const serviceName = getApiServiceName(dataModels[valueItem.serviceName]);
                stateFetch[serviceName] = stateFetch[serviceName] || [];
                stateFetch[serviceName].push(paramString);
              }
              else {
                stateValue[paramStateName] = stateValue[paramStateName] || [];
                stateValue[paramStateName].push(paramString);
              }
            })
            props += `${key}={{
              value: this.state.${stateName} && this.state.${stateName}.${key},
              name: '${isFormItem ? firstValue.keyName : ''}',
              rules: ${JSON.stringify(rules)},
              onChange: (value) => {
                ${Object.values(resultValues).join('; \n')}
                ${Object.keys(stateFetch).map(fetchItemName => {
                  return `this.${fetchItemName}({
                    ${stateFetch[fetchItemName] && stateFetch[fetchItemName].join(', \n')}
                  });`;
                }).join('; \n')}
                this.setState({
                  ${Object.keys(stateValue).map(stateValueItem => {
                    return `${stateValueItem}: {
                      ...(this.state.${stateValueItem} || {}),
                      ${stateValue[stateValueItem].join(', \n')}
                    }`
                  }).join(', \n')}
                  ${stateName}: {
                    ...(this.state.${stateName} || {}),
                    ${key}: value
                  }
                })
              }
            }}\n`;
          } else {
            props += `${key}={{
              value: this.state.${stateName} && this.state.${stateName}.${key},
              onChange: (value) => {
                this.setState({
                  ${stateName}: {
                    ...(this.state.${stateName} || {}),
                    ${key}: value
                  }
                });
              }
            }}\n`;
          }
        }
        else if(propsOptionsObj &&
          propsOptionsObj[key] &&
          propsOptionsObj[key].type === "img" && 
          value && value.length
        ) {
          value = value.map(item => {
            return replaceImgUrlInProps(item, this.imagesDirPath);
          })
          value = JSON.stringify(value);
          props += `${key}={${value}}\n`;
        }
        else if (
          propsOptionsObj &&
          propsOptionsObj[key] &&
          propsOptionsObj[key].type === "icon" &&
          propsOptionsObj[key].value && propsOptionsObj[key].value.src
        ) {
          const iconValue = propsOptionsObj[key].value;
          let iconPropsValue = 'null';

          let imgSrc = replaceImgUrlInProps(iconValue.src, this.imagesDirPath);
          
          if(imgSrc && imgSrc.match(/\.svg$/)) {
            iconPropsValue = `<Svg src={'${imgSrc}'} colorType={'${iconValue.colorType || ""}'} />`;
            if(!this.dependencies['@icode-run/svg']) {
              this.dependencies['@icode-run/svg'] = {
                remoteAppName: 'icodeRun',
                componentName: 'Svg',
                packageName: '@icode-run/svg',
              }
            }
          }
          else if(imgSrc) {
            iconPropsValue = `<img src={'${imgSrc}'} />`
          }
          
          props += `${key}={${iconPropsValue}}\n`;
        }
         else if (
          propsOptionsObj &&
          propsOptionsObj[key] &&
          propsOptionsObj[key].type === "modal"
        ) {
          props += `${key}={{
            value: this.state['${getApiStateName(item.nodeId, {relations, dataModels})}Visible'],
            onChange: (value) => {
              this.setState({
                ${getApiStateName(item.nodeId, {relations, dataModels})}Visible: value,
                ${relation.stateNodeId ? getApiStateName(relation.stateNodeId, {relations, dataModels}) + 'EventData: []': ''}
              })
            }
          }}\n`;
        } else if (
          propsOptionsObj &&
          propsOptionsObj[key] &&
          propsOptionsObj[key].type === "text"
        ) {
          textProps = propsOptionsObj[key];
        }else if (
          propsOptionsObj &&
          propsOptionsObj[key] &&
          propsOptionsObj[key].type === "javascript"
        ) {
          const fnName = getNewNodeId(item.nodeId);
          this.codeConfig[fnName] = value;
          props += `${key}={codeConfig.${fnName}}\n`;
          

          // .replace(/^\s*\`/, "")
          // .replace(/\`\s*$/, "")
        } else if (
          propsOptionsObj &&
          propsOptionsObj[key] &&
          propsOptionsObj[key].type === "data" &&
          value &&
          value.length
        ) {
          const dataValue = JSON.parse(JSON.stringify(value));
          let dataValueString = "";
          dataValue.forEach((valueItem) => {
            // tianyanrong
            if (valueItem.isRender) {
              const scopeName = `${getApiStateName(relation.nodeId, {relations, dataModels})}Scope`;
              dataValueString += `{
                title: '${valueItem.title}',
                keyName: '${valueItem.keyName}',
                isRender: true,
                render: (${scopeName}, data, index) => {
                  return (
                    <div style={{
                      
                      display: 'flex', 
                      flexDirection: 'column', 
                      height: '100%'}}>
                      ${this.getRenderHtml(item.children, scopeName, isFormItem)}
                    </div>
                  );
                },
              }`;
            } else {
              dataValueString += `${JSON.stringify(valueItem)},`;
            }
          });

          props += `${key}={[${dataValueString}]}\n`;
          props += `dataCount={this.state.${getApiStateName(item.nodeId, {relations, dataModels})}Count}\n`;
        } else if (typeof value === "object") {
          value = JSON.stringify(value);
          props += `${key}={${value}}\n`;
        } else if (typeof value === "string") {
          props += `${key}={'${value}'}\n`; //key + '={`' + value + '`}';
        } else {
          props += `${key}={${value}}\n`;
        }
      }
    }

    const dataPropsValue = this.getPropsData(item, isScope);
    if (dataPropsValue) {
      props += `data={${dataPropsValue}}\n`;
    }
    if(textProps && textProps.name) {
      const content = (textProps.extendValues && textProps.extendValues.content) || '';
      if(textProps.value && textProps.value.length && content && content.match(/\[\d+\]/g)) {
        props += `${textProps.name}={getTextContent(${dataPropsValue}, ${JSON.stringify(textProps.value.map(textValueItem => {
          return textValueItem.keyName;
        }))}, "${content}")}`;
        this.useDependencies.getTextContent = true;
      }
      else {
        props += `${textProps.name}={"${content}"}`;
      }
      
    }

    const isApi = relation.dataSourceType === 'api' && relation.dataSourceName && dataModels[relation.dataSourceName];
    if (isApi) {
      const serviceName = getApiServiceName(dataModels[relation.dataSourceName]);
      const from = dataModels[serviceName] && dataModels[serviceName].type;
      props += `setState={(payload) => {
        this.setState({
          ${getApiStateName(item.nodeId, {relations, dataModels, from})}Params: payload,
        })
      }}\n`;
      props += `onFetch={this.${serviceName}}\n`;
      props += `isLoading={this.state.${getApiStateName(
        item.nodeId, {relations, dataModels, from}
      )}Loading || false}\n`; 
    }

    // 收集refreshData配置数据
    const isRefresh = relation.refreshData && relation.refreshData.isOpen;
    if(relation.dataSourceType === 'static'  && isRefresh) {
      this.refreshData[item.nodeId] = {
        type: 'static',
        stateName: getStaticDataName(item.nodeId),
        refreshData: relation.refreshData
      }
    }
    else if(isApi && isRefresh) {
      this.refreshData[item.nodeId] = {
        type: 'api',
        serviceName: getApiServiceName(dataModels[relation.dataSourceName]),
        refreshData: relation.refreshData
      }
    }


    if(relation.submitDataModelName && dataModels[relation.submitDataModelName]) {
      props += `submitData={this.state.${getApiStateName(item.nodeId, {relations, dataModels, from: 'update'})}Params}`;
      props += `onSubmit={this.${getApiServiceName(dataModels[relation.submitDataModelName])}}\n`;
    }

    // props += `setGlobalState={this.setGlobalState}\n`;

    // // events编译
    // if(relation.events && relation.events.length) {
    //   const newEvents = getNewEvents({
    //     events: relation.events, 
    //     dataModels: dataModels,
    //     relations: relations,
    //     paramDefaultValueState: this.paramDefaultValueState,
    //   });
    //   const currentEvents = newEvents.events;
    //   let eventKey, eventsString = '';
    //   for(eventKey in currentEvents) {
    //     if(['onClick', 'onDoubleClick'].indexOf(eventKey) >= 0) {
    //       eventsString += `${eventKey}: () => {
    //         this.getEventCallBack(${JSON.stringify(currentEvents[eventKey])}${newEvents.stateNodeId ? ', {scopeData: ['+ newEvents.stateNodeId +'Scope]}' : ''});
    //       },\n`;
    //     }
    //   }

    //   props += `events={{
    //     ${eventsString}
    //   }
    //   }\n`
    // }

    props = props
      .replace(/"___/g, "")
      .replace(/___"/g, "")
      .replace(/\\n/g, "")
      .replace(/\\"/g, '"');
    return props;
  },

  updateComponentNodeId: function (componentConfig) {
    componentConfig.map((item) => {
      item.nodeId = setNewNodeId(item);

      if(item.children && item.children.length) {
        this.updateComponentNodeId(item.children)
      }
    });
  },

  getRenderHtml: function (componentConfig, isScope, isFormItem) {
    const relations = this.relations;
    const dataModels = this.dataModels;
    let html = "";
    componentConfig.map((item) => {
      const relation = getCurrentRelations(item.nodeId, this.relations);
      getParamDefaultValueState({
        paramStates: this.paramDefaultValueState, 
        propsOptions: item.propsOptions, 
        currRelation: relation,
        relations,
        dataModels,
      });

      // if(item.remoteEntryPath && item.remoteAppName) {
      //   this.remoteEntryPaths[item.remoteEntryPath] = {
      //     remoteEntryPath: item.remoteEntryPath,
      //     remoteAppName: item.remoteAppName,
      //   }
      // }
      

      // 是否可以渲染children
      let isCanRenderChildren = false;
      const isFormContainer = item.componentName === "Form" || isFormItem;
      if (item.componentName !== "EmptyContainer") {
       
        // 依赖
       if (!this.dependencies[item.packageName]) {
          this.dependencies[item.packageName] = {
            componentName: item.componentName,
            packageName: item.packageName,
            remoteAppName: item.remoteAppName,
          }
        }


        

        // 样式
        // if (item.style) {
        //   let style = {},
        //     key,
        //     oldStyle = item.style;
        //   let isHasStyle = false;
        //   for (key in oldStyle) {
        //     if (oldStyle[key]) {
        //       isHasStyle = true;
        //       style[this.camelToMiddleline(key)] = oldStyle[key];
        //     }
        //   }
        //   if (isHasStyle) {
        //     this.style[item.nodeId] = style;
        //   }
        // }
        // 自定义组件内部样式
        if(item.css) {
          let css = replaceImgUrlInCss(item.css, this.imagesDirPath);
          css = getCss(css);
          this.style[item.nodeId] = css['&'];
          this.css[`css-${item.nodeId}`] = css['global'];
        }

        // 重新编辑propsOptions数据成对像，方便查询
        const propsOptions = item.propsOptions;
        const newPropsOptions = {};

        propsOptions.map((option) => {
          newPropsOptions[option.name] = option;
          if (option.type === "param" && item.props) {
            item.props[option.name] = true;
          }
          else if(option.type === "modal" && item.props) {
            item.props[option.name] = true;
          }
          // if (option.type === "param" && option.value && option.value.length) {
          //   newPropsOptions.callbackService = option.value;
          //   if (option.staticProps && option.staticProps.isTriggerRequest) {
          //     newPropsOptions.isTriggerRequest = true;
          //   }
          // }
          // else
          if (option.type === "container") {
            isCanRenderChildren = true;
          }
        });
        this.propsOptions[item.nodeId] = newPropsOptions;
        this.propsValues[item.nodeId] = item.props;
      } else {
        isCanRenderChildren = true;
      }

      // 加上className,来源于style、css
      let classNameText = '`';
      if(this.style[item.nodeId]) {
        classNameText += '${styles["'+item.nodeId+'"]} ';
      }
      if(this.css[`css-${item.nodeId}`]) {
        classNameText += '${styles["css-'+item.nodeId+'"]}';
      }
      classNameText += '`';
      let className = this.style[item.nodeId] || this.css[`css-${item.nodeId}`]
        ? `className={${classNameText}}`
        : "";
      const themeMode = "global={this.state.global}";
      const props = this.getProps(item, isScope, isFormContainer);
      const isHadChildren = isCanRenderChildren && item.children && item.children.length;
      html += `${
        item.componentName !== "EmptyContainer"
          ? `<${item.componentName}
          ${className}
          ${themeMode}
          ${props}
        ${isHadChildren ? '' : '/'} ${item.componentName === 'Theme' ? 'style={{ backgroundColor: this.state.global && this.state.global.backgroundColor }}' : ''}>`
          : ""
      }
      ${
        isHadChildren ? this.getRenderHtml(item.children, isScope, isFormContainer) : ""
      }
      ${
        (item.componentName !== "EmptyContainer" && isHadChildren)
          ? `</${item.componentName}>`
          : ""
      }`;
      // import { parseLogic } from '@icode-run/utils/es/parseLogic';
      // if (relation.displayLogic) {
      //   const dataPropsValue = this.getPropsData(item, isScope);
      //   if (dataPropsValue) {
      //     html = `<ShowContainer
      //       isShow={parseLogic('', '${relation.displayLogic}', '', ${dataPropsValue}[0])}
      //     >
      //     ${html}
      //     </ShowContainer>`;
      //     this.dependencies['@icode-run/show-container'] = {
      //       componentName: 'ShowContainer',
      //       packageName: '@icode-run/show-container',
      //       remoteAppName: 'icodeRun',
      //     }
      //   }
      // }
      const displayLogic = relation.displayLogic || {};
      if (displayLogic.dataFrom && displayLogic.targetValues) {
        
        const dataFrom = displayLogic.dataFrom;
        const targetValues = displayLogic.targetValues || [];
        if(dataFrom && dataFrom[0] === 'state' && dataFrom[1] && dataFrom[2]) {
          const nodeStateName = `${getApiStateName(dataFrom[1], {relations, dataModels})}DefaultValue`;
          html = `<ShowContainer
            isShow={${JSON.stringify(targetValues)}.indexOf(this.${dataFrom[0]}.${nodeStateName} && this.${dataFrom[0]}.${nodeStateName}.${dataFrom[2]} + '') >= 0}
          >
          ${html}
          </ShowContainer>`;
          this.dependencies['@icode-run/show-container'] = {
            componentName: 'ShowContainer',
            packageName: '@icode-run/show-container',
            remoteAppName: 'icodeRun',
          }
        }
      }


      // events编译
      if(relation.events && relation.events.length) {
        const newEvents = getNewEvents({
          events: relation.events, 
          dataModels: dataModels,
          relations: relations,
          paramDefaultValueState: this.paramDefaultValueState,
        });
        const currentEvents = newEvents.events;
        let eventKey, eventProps = '';
        for(eventKey in currentEvents) {
          if(['onClick', 'onDoubleClick'].indexOf(eventKey) >= 0) {
            eventProps += `${eventKey}={() => {
              this.getEventCallBack(${JSON.stringify(currentEvents[eventKey])}${newEvents.stateNodeId ? ', {scopeData: ['+ newEvents.stateNodeId +'Scope]}' : ''});
            }}\n`;
            this.useDependencies.getEventCallBack = true;
          }
        }

        html = `<a ${eventProps}>${html}</a>`;
      }

    });

    return html;
  },
};

// 生成package.json
const createPackageFile = async function(data, config) {
  const packageDefaultPath = path.join(
    config.resourcesRoot,
    config.resourcesPath[0],
    config.resourcesPath[1] || "",
    config.resourcesPath[2] || "",
    data.appName,
    "package.json"
  );
  const packagePath = config.packagePath ? path.join(config.packagePath, "package.json") : packageDefaultPath;
  if (
    !fs.existsSync(packagePath)
  ) {
    const str = await ejs.renderFile(
      path.join(__dirname, "./template/package.ejs"),
      {
        appName: data.appName || '',
      },
      {}
    );
    await fs.writeFileSync(
      packagePath,
      str,
      {
        encoding: "utf-8",
      }
    );
  }
};

// 
module.exports = async function (content, config, isLoadingNpm) {
  try {
    const data = JSON.parse(content);

    const resourcesPath = path.join(
      config.resourcesRoot,
      config.resourcesPath[0],
      config.resourcesPath[1] || "",
      config.resourcesPath[2] || "",
      data.appName,
      data.pageName
    );
  
    await createDir(resourcesPath);
    await createPackageFile(data, config);

    new contentInfo({
      data, 
      resourcesPath, 
      imagesDirPath: config.imagesDirPath,
      globalDirPath: config.globalDirPath,
      packagePath: config.packagePath,
      isLoadingNpm,
    });
  } catch (e) {
    console.log("---------json-2-code error=", e);
  }
};
